from typing import Dict, List, Union

from .scraper import scrape_technologies, scrape_technologies_to_json


class Pywappalyzer:  # noqa
    def use_latest(self) -> None:
        scrape_technologies_to_json()

    def analyze(self, url: str) -> Dict[str, Union[str, Dict[str, List[str]]]]:
        technologies = scrape_technologies(url=url)
        return technologies



import json

import pytest


class Helpers:
    def read_json(self, filename: str) -> dict:
        with open(filename, "r") as f:
            return {
                key: value.replace("\\xa0", "\xa0")
                for key, value in json.load(f).items()
            }

    def txt_to_bytes(self, filename: str) -> bytes:
        with open(filename, "r") as f:
            return f.read().encode()

    def technologies_msg(self, filename: str):
        with open(filename, "r") as f:
            return json.load(f)

    def sort_dict_lists(self, value):
        return {key: sorted(val) for key, val in value.items()}


@pytest.fixture
def helpers():
    return Helpers()



import json
from pathlib import Path
from typing import Dict, List, Union

import requests
from bs4 import BeautifulSoup as bs

SITE_LINK = "https://w3techs.com/sitesinfo?url={}"
TECHNOLOGIES_LINK = "https://w3techs.com/technologies"
HEADERS = {
    "accept": "*/*",
    "user-agent": (
        "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 "
        "(KHTML, like Gecko) Chrome/85.0.4183.121 Safari/537.36"
    ),
}

FILE_PATH = Path(__file__).parent


def write_json(filename: str, data: dict) -> None:
    with open(filename, "w+") as f:
        json.dump(data, f, ensure_ascii=True, indent=4)


def read_technologies_json(filename: str) -> dict:
    with open(filename, "r") as f:
        data = json.load(f)

    technologies = {}
    for tech in list(data.keys()):
        element = data[tech]

        for el in element:
            technologies[el] = tech

    return technologies


def get_html(url: str) -> bytes:
    response: requests.Response = requests.get(url=url, headers=HEADERS)
    return response.content


def parse_technologies_html(content: bytes) -> List[str]:
    soup = bs(content, "lxml")
    bars = soup.find("table", attrs={"class": "bars"})
    trs = bars.find_all("tr")
    ul_techs = [tech.text for tech in soup.find("ul").find_all("li")]
    technologies = []

    for tr in trs[3:]:
        tech = tr.find_all("th")
        if len(tech) > 0:
            technologies.append(tech[0].text)

    return technologies + ul_techs


def scrape_technologies(url: str) -> Dict[str, Union[str, Dict[str, List[str]]]]:
    result: Dict[str, Union[str, Dict[str, List[str]]]] = {"technologies": {}}
    html = get_html(url=SITE_LINK.format(url))
    soup = bs(html, "lxml")
    technologies_json = read_technologies_json(f"{FILE_PATH}/db/technologies.json")
    tech_table = soup.find("td", attrs={"class": "tech_main"})
    technologies = tech_table.find_all("p", attrs={"class": "si_tech"})

    result["url"] = SITE_LINK.format(url)
    technology_names = {technology.find("a").text for technology in technologies}
    technologies_dict: Dict[str, List[str]] = result["technologies"]  # type: ignore

    for technology_name in technology_names:

        if not technologies_json.get(technology_name):
            continue

        if technologies_json[technology_name] in result["technologies"]:
            technologies_dict[technologies_json[technology_name]].append(
                technology_name
            )

        elif technologies_json[technology_name] not in result["technologies"]:
            technologies_dict[technologies_json[technology_name]] = [technology_name]

    return result


def scrape_technologies_to_json(
    filename: str = f"{FILE_PATH}/db/technologies.json",
) -> None:
    html = get_html(url=TECHNOLOGIES_LINK)
    soup = bs(html, "lxml")
    tech_sel = soup.find("td", attrs={"class": "tech_sel"})
    links = tech_sel.find_all("a")
    categories = {}

    for link in links:
        href = link.get("href")
        name = link.text.strip()
        tech_html = get_html(url=href)
        technologies = parse_technologies_html(content=tech_html)
        categories[name] = technologies

    write_json(filename, data=categories)



import pytest

from pywappalyzer.scraper import (  # isort:skip
    get_html,
    parse_technologies_html,
    read_technologies_json,
    scrape_technologies,
    scrape_technologies_to_json,
    write_json,
)


@pytest.fixture
def mock_open(mocker):
    mock_open = mocker.patch("pywappalyzer.scraper.open")
    mock_open.dump.return_value = None
    return mock_open


@pytest.fixture
def mock_requests(mocker, helpers):
    mock_requests = mocker.patch("pywappalyzer.scraper.requests")
    mock_requests.get.return_value.content = helpers.txt_to_bytes(
        filename="samples/get_html_sample.txt"
    )
    return mock_requests


def test_write_json(mock_open):
    assert write_json(filename="Some file", data={}) is None


def test_read_technologies_json(helpers):
    data = read_technologies_json(filename="samples/technologies.json")
    expected_out = helpers.read_json(filename="samples/technologies_list.json")

    assert data == expected_out


def test_get_html(mock_requests, helpers):
    data = get_html("https://w3techs.com/technologies/overview/content_management")

    assert data == helpers.txt_to_bytes("samples/get_html_sample.txt")


def test_parse_technologies_html(helpers):
    content = helpers.txt_to_bytes("samples/get_html_sample.txt")
    data = parse_technologies_html(content=content)

    assert data == helpers.technologies_msg(
        filename="samples/technologies_list_array.json"
    )


def test_scrape_technologies(mock_requests, helpers):
    mock_requests.get.return_value.content = helpers.txt_to_bytes(
        "samples/get_html_scrape_technologies.txt"
    )
    data = scrape_technologies(
        url="https://w3techs.com/sitesinfo?url=https://google.com"
    )

    assert helpers.sort_dict_lists(data) == helpers.sort_dict_lists(
        {
            "technologies": {
                "Image\xa0File\xa0Formats": ["SVG", "GIF", "PNG"],
                "Site\xa0Elements": ["Default protocol https"],
                "Email\xa0Servers": ["Fastmail", "Hetzner"],
                "Traffic\xa0Analysis\xa0Tools": ["Matomo"],
                "SSL\xa0Certificate\xa0Authorities": ["IdenTrust"],
                "Structured\xa0Data": ["JSON-LD"],
                "Social\xa0Widgets": ["Twitter", "LinkedIn"],
                "Client-side\xa0Languages": ["JavaScript"],
            },
            "url": "https://w3techs.com/sitesinfo?url=https://w3techs.com/sitesinfo?url=https://google.com",
        }
    )


def test_scrape_technologies_to_json(mock_open, mock_requests):
    assert scrape_technologies_to_json() is None



import pytest

from pywappalyzer.wappalyzer import Pywappalyzer


@pytest.fixture
def wappalyzer():
    return Pywappalyzer()


@pytest.fixture
def mock_scrape_technologies_to_json(mocker):
    mock_scrape_technologies_to_json = mocker.patch(
        "pywappalyzer.wappalyzer.scrape_technologies_to_json"
    )
    mock_scrape_technologies_to_json.return_value = None
    return mock_scrape_technologies_to_json


@pytest.fixture
def mock_scrape_technologies(mocker):
    mock_scrape_technologies = mocker.patch(
        "pywappalyzer.wappalyzer.scrape_technologies"
    )
    mock_scrape_technologies.return_value = {
        "technologies": {
            "Structured\xa0Data": ["Microdata", "JSON-LD"],
            "Site\xa0Elements": [
                "IPv6",
                "HTTP Strict Transport Security",
                "QUIC",
                "HTTP/2",
                "HTTP/3",
                "Default subdomain www",
                "Default protocol https",
            ],
            "Client-side\xa0Languages": ["JavaScript", "Flash"],
            "SSL\xa0Certificate\xa0Authorities": ["GlobalSign"],
            "Web\xa0Servers": ["Google Servers"],
            "Image\xa0File\xa0Formats": ["PNG", "SVG", "JPEG", "WebP"],
            "DNS\xa0Servers": ["Google"],
            "Advertising\xa0Networks": ["Google Ads"],
            "Traffic\xa0Analysis\xa0Tools": ["Google Analytics"],
        },
        "url": "https://w3techs.com/sitesinfo?url=https://google.com",
    }
    return mock_scrape_technologies


def test_use_latest(wappalyzer: Pywappalyzer, mock_scrape_technologies_to_json):
    assert wappalyzer.use_latest() is None  # type: ignore


def test_analyze(wappalyzer: Pywappalyzer, mock_scrape_technologies):
    data = wappalyzer.analyze(url="https://google.com")
    assert data == {
        "technologies": {
            "Structured\xa0Data": ["Microdata", "JSON-LD"],
            "Site\xa0Elements": [
                "IPv6",
                "HTTP Strict Transport Security",
                "QUIC",
                "HTTP/2",
                "HTTP/3",
                "Default subdomain www",
                "Default protocol https",
            ],
            "Client-side\xa0Languages": ["JavaScript", "Flash"],
            "SSL\xa0Certificate\xa0Authorities": ["GlobalSign"],
            "Web\xa0Servers": ["Google Servers"],
            "Image\xa0File\xa0Formats": ["PNG", "SVG", "JPEG", "WebP"],
            "DNS\xa0Servers": ["Google"],
            "Advertising\xa0Networks": ["Google Ads"],
            "Traffic\xa0Analysis\xa0Tools": ["Google Analytics"],
        },
        "url": "https://w3techs.com/sitesinfo?url=https://google.com",
    }
